@@ -2,6 +2,13 @@
 /* Copyright (c) 2015, NORDUnet A/S */
 /* See LICENSE for licensing information. */
 
+#ifndef _RADMSG_H
+#define _RADMSG_H
+
+#include "tlv11.h"
+
+#define RAD_Max_Attr_Value_Length 253
+
 #define RAD_Access_Request 1
 #define RAD_Access_Accept 2
 #define RAD_Access_Reject 3
@@ -29,9 +36,14 @@ struct radmsg {
     uint8_t code;
     uint8_t id;
     uint8_t auth[20];
-    struct list *attrs;
+    struct list *attrs; /*struct tlv*/
 };
 
+#define ATTRTYPE(x) ((x)[0])
+#define ATTRLEN(x) ((x)[1])
+#define ATTRVAL(x) ((x) + 2)
+#define ATTRVALLEN(x) ((x)[1] - 2)
+
 void radmsg_free(struct radmsg *);
 struct radmsg *radmsg_init(uint8_t, uint8_t, uint8_t *);
 int radmsg_add(struct radmsg *, struct tlv *);
@@ -40,8 +52,16 @@ struct list *radmsg_getalltype(const struct radmsg *msg, uint8_t type);
 int radmsg_copy_attrs(struct radmsg *dst,
                       const struct radmsg *src,
                       uint8_t type);
-uint8_t *radmsg2buf(struct radmsg *msg, uint8_t *);
-struct radmsg *buf2radmsg(uint8_t *, uint8_t *, uint8_t *);
+uint8_t *tlv2buf(uint8_t *p, const struct tlv *tlv);
+uint8_t *radmsg2buf(struct radmsg *msg, uint8_t *, int);
+struct radmsg *buf2radmsg(uint8_t *, uint8_t *, int, uint8_t *);
+uint8_t attrname2val(char *attrname);
+int vattrname2val(char *attrname, uint32_t *vendor, uint32_t *type);
+int attrvalidate(unsigned char *attrs, int length);
+struct tlv *makevendortlv(uint32_t vendor, struct tlv *attr);
+int resizeattr(struct tlv *attr, uint8_t newlen);
+
+#endif /*_RADMSG_H*/
 
 /* Local Variables: */
 /* c-file-style: "stroustrup" */
