@@ -84,6 +84,7 @@ extern int optind;
 extern char *optarg;
 #endif
 static const struct protodefs *protodefs[RAD_PROTOCOUNT];
+pthread_attr_t pthread_attr;
 
 /* minimum required declarations to avoid reordering code */
 struct realm *adddynamicrealmserver(struct realm *realm, char *id);
@@ -1648,8 +1649,11 @@ void *clientwr(void *arg) {
 #endif
 	pthread_mutex_unlock(&server->newrq_mutex);
 
-	for (i = 0; i < MAX_REQUESTS; i++) {
-	    if (server->clientrdgone) {
+    if (do_resend || server->lastrcv.tv_sec > laststatsrv.tv_sec)
+        statusserver_requested = 0;
+
+    for (i = 0; i < MAX_REQUESTS; i++) {
+        if (server->clientrdgone) {
 		server->state = RSP_SERVER_STATE_FAILING;
                 if (conf->pdef->connecter)
                     pthread_join(clientrdth, NULL);
@@ -1680,7 +1684,7 @@ void *clientwr(void *arg) {
             continue;
         }
 
-        if (rqout->tries > 0 && now.tv_sec - server->lastrcv.tv_sec > conf->retryinterval)
+        if (rqout->tries > 0 && now.tv_sec - server->lastrcv.tv_sec > conf->retryinterval && !do_resend)
             statusserver_requested = 1;
         if (rqout->tries == (*rqout->rq->buf == RAD_Status_Server ? 1 : conf->retrycount + 1)) {
             debug(DBG_DBG, "clientwr: removing expired packet from queue");
@@ -2322,7 +2326,7 @@ int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
 	    "host", CONF_MSTR, &conf->hostsrc,
             "IPv4Only", CONF_BLN, &ipv4only,
             "IPv6Only", CONF_BLN, &ipv6only,
-	    "secret", CONF_STR, &conf->confsecret,
+	    "secret", CONF_STR_NOESC, &conf->confsecret,
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
 	    "tls", CONF_STR, &conf->tls,
 	    "matchcertificateattribute", CONF_STR, &conf->matchcertattr,
@@ -2523,7 +2527,7 @@ int confserver_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
                           "IPv4Only", CONF_BLN, &ipv4only,
                           "IPv6Only", CONF_BLN, &ipv6only,
 			  "port", CONF_STR, &conf->portsrc,
-			  "secret", CONF_STR, &conf->confsecret,
+			  "secret", CONF_STR_NOESC, &conf->confsecret,
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
 			  "tls", CONF_STR, &conf->tls,
 			  "MatchCertificateAttribute", CONF_STR, &conf->matchcertattr,
@@ -2688,12 +2692,12 @@ int confrewrite_cb(struct gconffile **cf, void *arg, char *block, char *opt, cha
         "removeVendorAttribute", CONF_MSTR, &rmvattrs,
         "whitelistAttribute", CONF_MSTR, &wlattrs,
         "whitelistVendorAttribute", CONF_MSTR, &wlvattrs,
-        "addAttribute", CONF_MSTR, &addattrs,
-        "addVendorAttribute", CONF_MSTR, &addvattrs,
+        "addAttribute", CONF_MSTR_NOESC, &addattrs,
+        "addVendorAttribute", CONF_MSTR_NOESC, &addvattrs,
         "modifyAttribute", CONF_MSTR, &modattrs,
         "modifyVendorAttribute", CONF_MSTR, &modvattrs,
-        "supplementAttribute", CONF_MSTR, &supattrs,
-        "supplementVendorAttribute", CONF_MSTR, &supvattrs,
+        "supplementAttribute", CONF_MSTR_NOESC, &supattrs,
+        "supplementVendorAttribute", CONF_MSTR_NOESC, &supvattrs,
         NULL))
         debugx(1, DBG_ERR, "configuration error");
     addrewrite(val, whitelist_mode, whitelist_mode? wlattrs : rmattrs, whitelist_mode? wlvattrs : rmvattrs,
@@ -3042,7 +3046,8 @@ int radsecproxy_main(int argc, char **argv) {
     sigaddset(&sigset, SIGHUP);
     sigaddset(&sigset, SIGPIPE);
     pthread_sigmask(SIG_BLOCK, &sigset, NULL);
-    pthread_create(&sigth, &pthread_attr, sighandler, NULL);
+    if (pthread_create(&sigth, &pthread_attr, sighandler, NULL))
+        debugx(1, DBG_ERR, "pthread_create failed: sighandler");
 
     for (entry = list_first(srvconfs); entry; entry = list_next(entry)) {
 	srvconf = (struct clsrvconf *)entry->data;
