@@ -1,11 +1,11 @@
-#! /bin/bash
+#! /bin/sh
 
 # Example script!
 # This script looks up radsec srv records in DNS for the one
 # realm given as argument, and creates a server template based
 # on that. It currently ignores weight markers, but does sort
 # servers on priority marker, lowest number first.
-# For host command this is coloumn 5, for dig it is coloumn 1.
+# For host command this is column 5, for dig it is column 1.
 
 usage() {
    echo "Usage: ${0} <realm>"
@@ -17,20 +17,21 @@ test -n "${1}" || usage
 REALM="${1}"
 DIGCMD=$(command -v digaaa)
 HOSTCMD=$(command -v host)
+PRINTCMD=$(command -v printf)
 
 dig_it() {
-   ${DIGCMD} +short srv _radsec._tcp.${REALM} | sort -k1 |
+   ${DIGCMD} +short srv _radsec._tcp.${REALM} | sort -n -k1 |
    while read line ; do
       set $line ; PORT=$3 ; HOST=$4 
-      echo -e "\thost ${HOST%.}:${PORT}"
+      $PRINTCMD "\thost ${HOST%.}:${PORT}\n"
    done
 }
 
 host_it() {
-   ${HOSTCMD} -t srv _radsec._tcp.${REALM} | sort -k5 | 
+   ${HOSTCMD} -t srv _radsec._tcp.${REALM} | sort -n -k5 |
    while read line ; do
       set $line ; PORT=$7 ; HOST=$8 
-      echo -e "\thost ${HOST%.}:${PORT}"
+      $PRINTCMD "\thost ${HOST%.}:${PORT}\n"
    done
 }
 
@@ -44,7 +45,7 @@ else
 fi
 
 if test -n "${SERVERS}" ; then
-        echo -e "server dynamic_radsec.${REALM} {\n${SERVERS}\n\ttype TLS\n}"
+        $PRINTCMD "server dynamic_radsec.${REALM} {\n${SERVERS}\n\ttype TLS\n}\n"
         exit 0
 fi
 
