@@ -1,10 +1,6 @@
-/*
- * Copyright (C) 2006-2009 Stig Venaas <venaas@uninett.no>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- */
+/* Copyright (c) 2007-2009, UNINETT AS
+ * Copyright (c) 2012-2013, 2017, NORDUnet A/S */
+/* See LICENSE for licensing information. */
 
 #include <signal.h>
 #include <sys/socket.h>
@@ -18,13 +14,13 @@
 #endif
 #include <sys/time.h>
 #include <sys/types.h>
-#include <sys/select.h>
 #include <ctype.h>
 #include <sys/wait.h>
 #include <arpa/inet.h>
 #include <regex.h>
 #include <pthread.h>
 #include <assert.h>
+#include <errno.h>
 #include "radsecproxy.h"
 #include "hostport.h"
 
@@ -110,10 +106,12 @@ static int addr_equal(struct sockaddr *a, struct sockaddr *b) {
     case AF_INET:
 	return !memcmp(&((struct sockaddr_in*)a)->sin_addr,
 		       &((struct sockaddr_in*)b)->sin_addr,
-		       sizeof(struct in_addr));
+		       sizeof(struct in_addr)) &&
+               (((struct sockaddr_in*)a)->sin_port == ((struct sockaddr_in*)b)->sin_port);
     case AF_INET6:
 	return IN6_ARE_ADDR_EQUAL(&((struct sockaddr_in6*)a)->sin6_addr,
-				  &((struct sockaddr_in6*)b)->sin6_addr);
+				  &((struct sockaddr_in6*)b)->sin6_addr) &&
+                  (((struct sockaddr_in6*)a)->sin6_port == ((struct sockaddr_in6*)b)->sin6_port);
     default:
 	/* Must not reach */
 	return 0;
@@ -133,7 +131,7 @@ uint16_t port_get(struct sockaddr *sa) {
 /* exactly one of client and server must be non-NULL */
 /* return who we received from in *client or *server */
 /* return from in sa if not NULL */
-unsigned char *radudpget(int s, struct client **client, struct server **server, uint16_t *port) {
+unsigned char *radudpget(int s, struct client **client, struct server **server) {
     int cnt, len;
     unsigned char buf[4], *rad = NULL;
     struct sockaddr_storage from;
@@ -141,105 +139,103 @@ unsigned char *radudpget(int s, struct client **client, struct server **server,
     socklen_t fromlen = sizeof(from);
     struct clsrvconf *p;
     struct list_node *node;
-    fd_set readfds;
     struct client *c = NULL;
     struct timeval now;
+    char tmp[INET6_ADDRSTRLEN];
 
     for (;;) {
-	if (rad) {
-	    free(rad);
-	    rad = NULL;
-	}
-	FD_ZERO(&readfds);
-        FD_SET(s, &readfds);
-	if (select(s + 1, &readfds, NULL, NULL, NULL) < 1)
-	    continue;
-	cnt = recvfrom(s, buf, 4, MSG_PEEK | MSG_TRUNC, (struct sockaddr *)&from, &fromlen);
-	if (cnt == -1) {
-	    debug(DBG_WARN, "radudpget: recv failed");
-	    continue;
-	}
+        if (rad) {
+            free(rad);
+            rad = NULL;
+        }
 
-	p = client
-	    ? find_clconf(handle, (struct sockaddr *)&from, NULL)
-	    : find_srvconf(handle, (struct sockaddr *)&from, NULL);
-	if (!p) {
-	    debug(DBG_WARN, "radudpget: got packet from wrong or unknown UDP peer %s, ignoring", addr2string((struct sockaddr *)&from));
-	    recv(s, buf, 4, 0);
-	    continue;
-	}
+        cnt = recvfrom(s, buf, 4, MSG_PEEK | MSG_TRUNC, (struct sockaddr *)&from, &fromlen);
+        if (cnt == -1) {
+            debug(DBG_ERR, "radudpget: recv failed - %s", strerror(errno));
+            continue;
+        }
 
-	len = RADLEN(buf);
-	if (len < 20) {
-	    debug(DBG_WARN, "radudpget: length too small");
-	    recv(s, buf, 4, 0);
-	    continue;
-	}
+        p = client
+            ? find_clconf(handle, (struct sockaddr *)&from, NULL)
+            : find_srvconf(handle, (struct sockaddr *)&from, NULL);
+        if (!p) {
+            debug(DBG_WARN, "radudpget: got packet from wrong or unknown UDP peer %s, ignoring", addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)));
+            if (recv(s, buf, 4, 0) == -1)
+                debug(DBG_ERR, "radudpget: recv failed - %s", strerror(errno));
+            continue;
+        }
 
-	rad = malloc(len);
-	if (!rad) {
-	    debug(DBG_ERR, "radudpget: malloc failed");
-	    recv(s, buf, 4, 0);
-	    continue;
-	}
+        len = RADLEN(buf);
+        if (len < 20) {
+            debug(DBG_WARN, "radudpget: length too small");
+            if (recv(s, buf, 4, 0) == -1)
+                debug(DBG_ERR, "radudpget: recv failed - %s", strerror(errno));
+            continue;
+        }
 
-	cnt = recv(s, rad, len, MSG_TRUNC);
-	debug(DBG_DBG, "radudpget: got %d bytes from %s", cnt, addr2string((struct sockaddr *)&from));
+        rad = malloc(len);
+        if (!rad) {
+            debug(DBG_ERR, "radudpget: malloc failed");
+            if (recv(s, buf, 4, 0) == -1)
+                debug(DBG_ERR, "radudpget: recv failed - %s", strerror(errno));
+            continue;
+        }
 
-	if (cnt < len) {
-	    debug(DBG_WARN, "radudpget: packet smaller than length field in radius header");
-	    continue;
-	}
-	if (cnt > len)
-	    debug(DBG_DBG, "radudpget: packet was padded with %d bytes", cnt - len);
+        cnt = recv(s, rad, len, MSG_TRUNC);
+        debug(DBG_DBG, "radudpget: got %d bytes from %s", cnt, addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)));
 
-	if (client) {
-	    *client = NULL;
-	    pthread_mutex_lock(p->lock);
-	    for (node = list_first(p->clients); node;) {
-		c = (struct client *)node->data;
-		node = list_next(node);
-		if (s != c->sock)
-		    continue;
-		gettimeofday(&now, NULL);
-		if (!*client && addr_equal((struct sockaddr *)&from, c->addr)) {
-		    c->expiry = now.tv_sec + 60;
-		    *client = c;
-		}
-		if (c->expiry >= now.tv_sec)
-		    continue;
+        if (cnt < len) {
+            debug(DBG_WARN, "radudpget: packet smaller than length field in radius header");
+            continue;
+        }
+        if (cnt > len)
+            debug(DBG_DBG, "radudpget: packet was padded with %d bytes", cnt - len);
 
-		debug(DBG_DBG, "radudpget: removing expired client (%s)", addr2string(c->addr));
-		removeudpclientfromreplyq(c);
-		c->replyq = NULL; /* stop removeclient() from removing common udp replyq */
-		removelockedclient(c);
-		break;
-	    }
-	    if (!*client) {
-		fromcopy = addr_copy((struct sockaddr *)&from);
-		if (!fromcopy) {
-		    pthread_mutex_unlock(p->lock);
-		    continue;
-		}
-		c = addclient(p, 0);
-		if (!c) {
-		    free(fromcopy);
-		    pthread_mutex_unlock(p->lock);
-		    continue;
-		}
-		c->sock = s;
-		c->addr = fromcopy;
-		gettimeofday(&now, NULL);
-		c->expiry = now.tv_sec + 60;
-		*client = c;
-	    }
-	    pthread_mutex_unlock(p->lock);
-	} else if (server)
-	    *server = p->servers;
-	break;
+        if (client) {
+            *client = NULL;
+            pthread_mutex_lock(p->lock);
+            for (node = list_first(p->clients); node;) {
+                c = (struct client *)node->data;
+                node = list_next(node);
+                if (s != c->sock)
+                    continue;
+                gettimeofday(&now, NULL);
+                if (!*client && addr_equal((struct sockaddr *)&from, c->addr)) {
+                    c->expiry = now.tv_sec + 60;
+                    *client = c;
+                }
+                if (c->expiry >= now.tv_sec)
+                    continue;
+
+                debug(DBG_DBG, "radudpget: removing expired client (%s)", addr2string(c->addr, tmp, sizeof(tmp)));
+                removeudpclientfromreplyq(c);
+                c->replyq = NULL; /* stop removeclient() from removing common udp replyq */
+                removelockedclient(c);
+                break;
+            }
+            if (!*client) {
+                fromcopy = addr_copy((struct sockaddr *)&from);
+                if (!fromcopy) {
+                    pthread_mutex_unlock(p->lock);
+                    continue;
+                }
+                c = addclient(p, 0);
+                if (!c) {
+                    free(fromcopy);
+                    pthread_mutex_unlock(p->lock);
+                    continue;
+                }
+                c->sock = s;
+                c->addr = fromcopy;
+                gettimeofday(&now, NULL);
+                c->expiry = now.tv_sec + 60;
+                *client = c;
+            }
+            pthread_mutex_unlock(p->lock);
+        } else if (server)
+            *server = p->servers;
+        break;
     }
-    if (port)
-	*port = port_get((struct sockaddr *)&from);
     return rad;
 }
 
@@ -247,11 +243,12 @@ int clientradputudp(struct server *server, unsigned char *rad) {
     size_t len;
     struct clsrvconf *conf = server->conf;
     struct addrinfo *ai;
+    char tmp[INET6_ADDRSTRLEN];
 
     len = RADLEN(rad);
     ai = ((struct hostportres *)list_first(conf->hostports)->data)->addrinfo;
     if (sendto(server->sock, rad, len, 0, ai->ai_addr, ai->ai_addrlen) >= 0) {
-	debug(DBG_DBG, "clienradputudp: sent UDP of length %d to %s port %d", len, addr2string(ai->ai_addr), port_get(ai->ai_addr));
+	debug(DBG_DBG, "clienradputudp: sent UDP of length %d to %s port %d", len, addr2string(ai->ai_addr, tmp, sizeof(tmp)), port_get(ai->ai_addr));
 	return 1;
     }
 
@@ -266,7 +263,7 @@ void *udpclientrd(void *arg) {
 
     for (;;) {
 	server = NULL;
-	buf = radudpget(*s, NULL, &server, NULL);
+	buf = radudpget(*s, NULL, &server);
 	replyh(server, buf);
     }
 }
@@ -281,8 +278,9 @@ void *udpserverrd(void *arg) {
 	    sleep(5); /* malloc failed */
 	    continue;
 	}
-	rq->buf = radudpget(*sp, &rq->from, NULL, &rq->udpport);
+	rq->buf = radudpget(*sp, &rq->from, NULL);
 	rq->udpsock = *sp;
+    gettimeofday(&rq->created, NULL);
 	radsrv(rq);
     }
     free(sp);
@@ -306,7 +304,6 @@ void *udpserverwr(void *arg) {
 	    memcpy(&to, reply->from->addr, SOCKADDRP_SIZE(reply->from->addr));
 	pthread_mutex_unlock(&replyq->mutex);
 	if (reply->from) {
-	    port_set((struct sockaddr *)&to, reply->udpport);
 	    if (sendto(reply->udpsock, reply->replybuf, RADLEN(reply->replybuf), 0, (struct sockaddr *)&to, SOCKADDR_SIZE(to)) < 0)
 		debug(DBG_WARN, "udpserverwr: send failed");
 	}
@@ -324,7 +321,7 @@ void addserverextraudp(struct clsrvconf *conf) {
     switch (((struct hostportres *)list_first(conf->hostports)->data)->addrinfo->ai_family) {
     case AF_INET:
 	if (client4_sock < 0) {
-	    client4_sock = bindtoaddr(srcres, AF_INET, 0, 1);
+	    client4_sock = bindtoaddr(srcres, AF_INET, 0);
 	    if (client4_sock < 0)
 		debugx(1, DBG_ERR, "addserver: failed to create client socket for server %s", conf->name);
 	}
@@ -332,7 +329,7 @@ void addserverextraudp(struct clsrvconf *conf) {
 	break;
     case AF_INET6:
 	if (client6_sock < 0) {
-	    client6_sock = bindtoaddr(srcres, AF_INET6, 0, 1);
+	    client6_sock = bindtoaddr(srcres, AF_INET6, 0);
 	    if (client6_sock < 0)
 		debugx(1, DBG_ERR, "addserver: failed to create client socket for server %s", conf->name);
 	}
@@ -352,15 +349,15 @@ void initextraudp() {
     }
 
     if (client4_sock >= 0)
-	if (pthread_create(&cl4th, NULL, udpclientrd, (void *)&client4_sock))
+	if (pthread_create(&cl4th, &pthread_attr, udpclientrd, (void *)&client4_sock))
 	    debugx(1, DBG_ERR, "pthread_create failed");
     if (client6_sock >= 0)
-	if (pthread_create(&cl6th, NULL, udpclientrd, (void *)&client6_sock))
+	if (pthread_create(&cl6th, &pthread_attr, udpclientrd, (void *)&client6_sock))
 	    debugx(1, DBG_ERR, "pthread_create failed");
 
     if (find_clconf_type(handle, NULL)) {
 	server_replyq = newqueue();
-	if (pthread_create(&srvth, NULL, udpserverwr, (void *)server_replyq))
+	if (pthread_create(&srvth, &pthread_attr, udpserverwr, (void *)server_replyq))
 	    debugx(1, DBG_ERR, "pthread_create failed");
     }
 }
